@@ -48,22 +48,22 @@ use crate::{
     http::curl::CurlBuilder,
     render::TemplateContext,
 };
-use anyhow::Context;
-use bytes::Bytes;
+use bytes::{Bytes, BytesMut};
 use chrono::Utc;
 use futures::{
-    Future,
+    Future, StreamExt, TryFutureExt, TryStreamExt,
     future::{self, OptionFuture, try_join_all},
+    stream::BoxStream,
     try_join,
 };
 use reqwest::{
-    Client, RequestBuilder, Response, Url,
+    Body, Client, RequestBuilder, Response, Url,
     header::{HeaderMap, HeaderName, HeaderValue},
     multipart::{Form, Part},
     redirect,
 };
 use slumber_config::HttpEngineConfig;
-use slumber_template::Template;
+use slumber_template::{RenderError, StreamSource, Template};
 use slumber_util::ResultTraced;
 use std::{collections::HashSet, error::Error};
 use tracing::{error, info, info_span};
@@ -134,7 +134,7 @@ impl HttpEngine {
     pub async fn build(
         &self,
         seed: RequestSeed,
-        template_context: &TemplateContext,
+        context: &TemplateContext,
     ) -> Result<RequestTicket, RequestBuildError> {
         let RequestSeed {
             id,
@@ -146,18 +146,23 @@ impl HttpEngine {
                 .entered();
 
         let future = async {
-            let recipe = template_context
-                .collection
-                .recipes
-                .try_get_recipe(recipe_id)?;
+            let recipe =
+                context.collection.recipes.try_get_recipe(recipe_id)?;
 
             // Render everything up front so we can parallelize it
             let (url, query, headers, authentication, body) = try_join!(
-                recipe.render_url(template_context),
-                recipe.render_query(options, template_context),
-                recipe.render_headers(options, template_context),
-                recipe.render_authentication(options, template_context),
-                recipe.render_body(options, template_context),
+                recipe.render_url(options, context),
+                recipe.render_query(options, context),
+                recipe.render_headers(options, context),
+                recipe.render_authentication(options, context),
+                // Body *has* to go last. Bodies are the only component that
+                // can be streamed. If a profile field is present in both the
+                // body and elsewhere, it should *never* be streamed. By
+                // starting every other component first, we ensure the body
+                // will never be the one to initiate the render for a multi-use
+                // profile field, meaning it won't get to render as a stream.
+                // This is kinda fragile but it's also a rare use case.
+                recipe.render_body(options, context),
             )?;
 
             // Build the reqwest request first, so we can have it do all the
@@ -168,7 +173,7 @@ impl HttpEngine {
             let mut builder =
                 client.request(recipe.method.into(), url).query(&query);
             if let Some(body) = body {
-                builder = body.apply(builder);
+                builder = body.apply(builder).await?;
             }
             // Set headers *after* body so the use can override the Content-Type
             // header that was set if they want to
@@ -180,13 +185,12 @@ impl HttpEngine {
             let request = builder.build()?;
             Ok((client, request))
         };
-        let (client, request) =
-            seed.run_future(future, template_context).await?;
+        let (client, request) = seed.run_future(future, context).await?;
 
         Ok(RequestTicket {
             record: RequestRecord::new(
                 seed,
-                template_context.selected_profile.clone(),
+                context.selected_profile.clone(),
                 &request,
                 self.large_body_size,
             )
@@ -200,7 +204,7 @@ impl HttpEngine {
     pub async fn build_url(
         &self,
         seed: RequestSeed,
-        template_context: &TemplateContext,
+        context: &TemplateContext,
     ) -> Result<Url, RequestBuildError> {
         let RequestSeed {
             id,
@@ -212,15 +216,13 @@ impl HttpEngine {
                 .entered();
 
         let future = async {
-            let recipe = template_context
-                .collection
-                .recipes
-                .try_get_recipe(recipe_id)?;
+            let recipe =
+                context.collection.recipes.try_get_recipe(recipe_id)?;
 
             // Parallelization!
             let (url, query) = try_join!(
-                recipe.render_url(template_context),
-                recipe.render_query(options, template_context),
+                recipe.render_url(options, context),
+                recipe.render_query(options, context),
             )?;
 
             // Use RequestBuilder so we can offload the handling of query params
@@ -231,7 +233,7 @@ impl HttpEngine {
                 .build()?;
             Ok(request)
         };
-        let request = seed.run_future(future, template_context).await?;
+        let request = seed.run_future(future, context).await?;
 
         Ok(request.url().clone())
     }
@@ -240,7 +242,7 @@ impl HttpEngine {
     pub async fn build_body(
         &self,
         seed: RequestSeed,
-        template_context: &TemplateContext,
+        context: &TemplateContext,
     ) -> Result<Option<Bytes>, RequestBuildError> {
         let RequestSeed {
             id,
@@ -252,14 +254,10 @@ impl HttpEngine {
                 .entered();
 
         let future = async {
-            let recipe = template_context
-                .collection
-                .recipes
-                .try_get_recipe(recipe_id)?;
+            let recipe =
+                context.collection.recipes.try_get_recipe(recipe_id)?;
 
-            let Some(body) =
-                recipe.render_body(options, template_context).await?
-            else {
+            let Some(body) = recipe.render_body(options, context).await? else {
                 return Ok(None);
             };
 
@@ -267,6 +265,14 @@ impl HttpEngine {
                 // If we have the bytes, we don't need to bother building a
                 // request
                 RenderedBody::Raw(bytes) => Ok(Some(bytes)),
+                RenderedBody::Stream(stream) => {
+                    let bytes = stream
+                        .stream
+                        .try_collect::<BytesMut>()
+                        .await
+                        .map_err(RequestBuildErrorKind::BodyStream)?;
+                    Ok(Some(bytes.into()))
+                }
 
                 // The body is complex - offload the hard work to RequestBuilder
                 RenderedBody::Json(_)
@@ -275,7 +281,7 @@ impl HttpEngine {
                     let url = Url::parse("http://localhost").unwrap();
                     let client = self.get_client(&url);
                     let mut builder = client.request(reqwest::Method::GET, url);
-                    builder = body.apply(builder);
+                    builder = body.apply(builder).await?;
                     let request = builder.build()?;
                     // We just added a body so we know it's present, and we
                     // know it's not a stream. This requires a clone which sucks
@@ -292,7 +298,7 @@ impl HttpEngine {
                 }
             }
         };
-        seed.run_future(future, template_context).await
+        seed.run_future(future, context).await
     }
 
     /// Render a recipe into a cURL command that will execute the request.
@@ -304,7 +310,7 @@ impl HttpEngine {
     pub async fn build_curl(
         &self,
         seed: RequestSeed,
-        template_context: &TemplateContext,
+        context: &TemplateContext,
     ) -> Result<String, RequestBuildError> {
         let RequestSeed {
             id,
@@ -316,18 +322,16 @@ impl HttpEngine {
                 .entered();
 
         let future = async {
-            let recipe = template_context
-                .collection
-                .recipes
-                .try_get_recipe(recipe_id)?;
+            let recipe =
+                context.collection.recipes.try_get_recipe(recipe_id)?;
 
             // Render everything up front so we can parallelize it
             let (url, query, headers, authentication, body) = try_join!(
-                recipe.render_url(template_context),
-                recipe.render_query(options, template_context),
-                recipe.render_headers(options, template_context),
-                recipe.render_authentication(options, template_context),
-                recipe.render_body(options, template_context),
+                recipe.render_url(options, context),
+                recipe.render_query(options, context),
+                recipe.render_headers(options, context),
+                recipe.render_authentication(options, context),
+                recipe.render_body(options, context),
             )?;
 
             // Buidl the command
@@ -338,11 +342,11 @@ impl HttpEngine {
                 builder = builder.authentication(&authentication);
             }
             if let Some(body) = body {
-                builder = builder.body(&body)?;
+                builder = builder.body(body).await?;
             }
             Ok(builder.build())
         };
-        seed.run_future(future, template_context).await
+        seed.run_future(future, context).await
     }
 
     /// Get the appropriate client to use for this request. If the request URL's
@@ -367,18 +371,21 @@ impl RequestSeed {
     /// Run the given future and convert any error into [RequestBuildError]
     async fn run_future<T>(
         &self,
-        future: impl Future<Output = anyhow::Result<T>>,
-        template_context: &TemplateContext,
+        future: impl Future<Output = Result<T, RequestBuildErrorKind>>,
+        context: &TemplateContext,
     ) -> Result<T, RequestBuildError> {
         let start_time = Utc::now();
-        future.await.traced().map_err(|error| RequestBuildError {
-            profile_id: template_context.selected_profile.clone(),
-            recipe_id: self.recipe_id.clone(),
-            id: self.id,
-            start_time,
-            end_time: Utc::now(),
-            source: error,
-        })
+        future
+            .await
+            .map_err(|error| RequestBuildError {
+                profile_id: context.selected_profile.clone(),
+                recipe_id: self.recipe_id.clone(),
+                id: self.id,
+                start_time,
+                end_time: Utc::now(),
+                error,
+            })
+            .traced()
     }
 }
 
@@ -429,7 +436,7 @@ impl RequestTicket {
                 request: self.record,
                 start_time,
                 end_time,
-                error: error.into(),
+                error,
             })
             .inspect_err(|err| error!(error = err as &dyn Error)),
         }
@@ -466,23 +473,24 @@ impl Recipe {
     /// Render base URL, *excluding* query params
     async fn render_url(
         &self,
-        template_context: &TemplateContext,
-    ) -> anyhow::Result<Url> {
-        let url = self
-            .url
-            .render_string(template_context)
+        options: &BuildOptions,
+        context: &TemplateContext,
+    ) -> Result<Url, RequestBuildErrorKind> {
+        let template = options.url.as_ref().unwrap_or(&self.url);
+        let url = template
+            .render_string(&context.streaming(false))
             .await
-            .context("Rendering URL")?;
+            .map_err(RequestBuildErrorKind::UrlRender)?;
         url.parse::<Url>()
-            .with_context(|| format!("Invalid URL: `{url}`"))
+            .map_err(|error| RequestBuildErrorKind::UrlInvalid { url, error })
     }
 
     /// Render query key=value params
     async fn render_query(
         &self,
         options: &BuildOptions,
-        template_context: &TemplateContext,
-    ) -> anyhow::Result<Vec<(String, String)>> {
+        context: &TemplateContext,
+    ) -> Result<Vec<(String, String)>, RequestBuildErrorKind> {
         let iter =
             self.query_iter().enumerate().filter_map(|(i, (k, _, v))| {
                 // Look up and apply override. We do this by index because the
@@ -490,15 +498,16 @@ impl Recipe {
                 let template = options.query_parameters.get(i, v)?;
 
                 Some(async move {
-                    Ok::<_, anyhow::Error>((
-                        k.to_owned(),
-                        template
-                            .render_string(template_context)
-                            .await
-                            .context(format!(
-                                "Rendering query parameter `{k}`"
-                            ))?,
-                    ))
+                    let value = template
+                        .render_string(&context.streaming(false))
+                        .await
+                        .map_err(|error| {
+                            RequestBuildErrorKind::QueryRender {
+                                parameter: k.to_owned(),
+                                error,
+                            }
+                        })?;
+                    Ok((k.to_owned(), value))
                 })
             });
         future::try_join_all(iter).await
@@ -509,8 +518,8 @@ impl Recipe {
     async fn render_headers(
         &self,
         options: &BuildOptions,
-        template_context: &TemplateContext,
-    ) -> anyhow::Result<HeaderMap> {
+        context: &TemplateContext,
+    ) -> Result<HeaderMap, RequestBuildErrorKind> {
         let mut headers = HeaderMap::new();
 
         // Render headers in an iterator so we can parallelize
@@ -521,7 +530,7 @@ impl Recipe {
                 let template = options.headers.get(i, value_template)?;
 
                 Some(async move {
-                    self.render_header(template_context, header, template).await
+                    self.render_header(context, header, template).await
                 })
             },
         );
@@ -540,14 +549,17 @@ impl Recipe {
     /// Render a single key/value header
     async fn render_header(
         &self,
-        template_context: &TemplateContext,
+        context: &TemplateContext,
         header: &str,
         value_template: &Template,
-    ) -> anyhow::Result<(HeaderName, HeaderValue)> {
+    ) -> Result<(HeaderName, HeaderValue), RequestBuildErrorKind> {
         let mut value: Vec<u8> = value_template
-            .render_bytes(template_context)
+            .render_bytes(&context.streaming(false))
             .await
-            .context(format!("Rendering header `{header}`"))?
+            .map_err(|error| RequestBuildErrorKind::HeaderRender {
+                header: header.to_owned(),
+                error,
+            })?
             .into();
 
         // Strip leading/trailing line breaks because they're going to trigger a
@@ -557,16 +569,20 @@ impl Recipe {
         // left in for backward compatibility.
         trim_bytes(&mut value, |c| c == b'\n' || c == b'\r');
 
-        // String -> header conversions are fallible, if headers
-        // are invalid
-        Ok::<(HeaderName, HeaderValue), anyhow::Error>((
-            header
-                .try_into()
-                .context(format!("Error encoding header name `{header}`"))?,
-            value.try_into().context(format!(
-                "Error encoding value for header `{header}`"
-            ))?,
-        ))
+        // String -> header conversions are fallible
+        let name: HeaderName = header.try_into().map_err(|error| {
+            RequestBuildErrorKind::HeaderInvalidName {
+                header: header.to_owned(),
+                error,
+            }
+        })?;
+        let value: HeaderValue = value.try_into().map_err(|error| {
+            RequestBuildErrorKind::HeaderInvalidValue {
+                header: header.to_owned(),
+                error,
+            }
+        })?;
+        Ok((name, value))
     }
 
     /// Render authentication and return the same data structure, with resolved
@@ -574,38 +590,37 @@ impl Recipe {
     async fn render_authentication(
         &self,
         options: &BuildOptions,
-        template_context: &TemplateContext,
-    ) -> anyhow::Result<Option<Authentication<String>>> {
+        context: &TemplateContext,
+    ) -> Result<Option<Authentication<String>>, RequestBuildErrorKind> {
         let authentication = options
             .authentication
             .as_ref()
             .or(self.authentication.as_ref());
+        let context = context.streaming(false); // Auth templates never support streaming
         match authentication {
             Some(Authentication::Basic { username, password }) => {
-                let (username, password) = try_join!(
-                    async {
+                let (username, password) =
+                    try_join!(
                         username
-                            .render_string(template_context)
+                            .render_string(&context)
+                            .map_err(RequestBuildErrorKind::AuthUsernameRender),
+                        async {
+                            OptionFuture::from(password.as_ref().map(
+                                |password| password.render_string(&context),
+                            ))
                             .await
-                            .context("Rendering username")
-                    },
-                    async {
-                        OptionFuture::from(password.as_ref().map(|password| {
-                            password.render_string(template_context)
-                        }))
-                        .await
-                        .transpose()
-                        .context("Rendering password")
-                    },
-                )?;
+                            .transpose()
+                            .map_err(RequestBuildErrorKind::AuthPasswordRender)
+                        },
+                    )?;
                 Ok(Some(Authentication::Basic { username, password }))
             }
 
             Some(Authentication::Bearer { token }) => {
                 let token = token
-                    .render_string(template_context)
+                    .render_string(&context)
                     .await
-                    .context("Rendering bearer token")?;
+                    .map_err(RequestBuildErrorKind::AuthTokenRender)?;
                 Ok(Some(Authentication::Bearer { token }))
             }
             None => Ok(None),
@@ -616,22 +631,33 @@ impl Recipe {
     async fn render_body(
         &self,
         options: &BuildOptions,
-        template_context: &TemplateContext,
-    ) -> anyhow::Result<Option<RenderedBody>> {
+        context: &TemplateContext,
+    ) -> Result<Option<RenderedBody>, RequestBuildErrorKind> {
         let Some(body) = options.body.as_ref().or(self.body.as_ref()) else {
             return Ok(None);
         };
 
         let rendered = match body {
+            // Raw body is always eagerly rendered
             RecipeBody::Raw(body) => RenderedBody::Raw(
-                body.render_bytes(template_context)
+                body.render_bytes(&context.streaming(false))
                     .await
-                    .context("Rendering body")?,
+                    .map_err(RequestBuildErrorKind::BodyRender)?,
             ),
+            // Stream body is rendered as a stream (!!)
+            RecipeBody::Stream(body) => {
+                let output = body.render(&context.streaming(true)).await;
+                let source = output.stream_source().cloned();
+                let stream = output
+                    .try_into_stream()
+                    .map_err(RequestBuildErrorKind::BodyRender)?
+                    .boxed();
+                RenderedBody::Stream(BodyStream { stream, source })
+            }
             RecipeBody::Json(json) => RenderedBody::Json(
-                json.render(template_context)
+                json.render(context)
                     .await
-                    .context("Rendering body")?,
+                    .map_err(RequestBuildErrorKind::BodyRender)?,
             ),
             RecipeBody::FormUrlencoded(fields) => {
                 let iter = fields.iter().enumerate().filter_map(
@@ -640,12 +666,18 @@ impl Recipe {
                             options.form_fields.get(i, value_template)?;
                         Some(async move {
                             let value = template
-                                .render_string(template_context)
+                                .render_string(&context.streaming(false))
                                 .await
-                                .context(format!(
-                                    "Rendering form field `{field}`"
-                                ))?;
-                            Ok::<_, anyhow::Error>((field.clone(), value))
+                                .map_err(|error| {
+                                    RequestBuildErrorKind::BodyFormFieldRender {
+                                        field: field.clone(),
+                                        error,
+                                    }
+                                })?;
+                            Ok::<_, RequestBuildErrorKind>((
+                                field.clone(),
+                                value,
+                            ))
                         })
                     },
                 );
@@ -654,18 +686,31 @@ impl Recipe {
             }
             RecipeBody::FormMultipart(fields) => {
                 let iter = fields.iter().enumerate().filter_map(
-                    |(i, (field, value_template))| {
+                    move |(i, (field, value_template))| {
                         let template =
                             options.form_fields.get(i, value_template)?;
                         Some(async move {
-                            let value = template
-                                .render_bytes(template_context)
-                                .await
-                                .context(format!(
-                                    "Rendering form field `{field}`"
-                                ))?
-                                .into();
-                            Ok::<_, anyhow::Error>((field.clone(), value))
+                            let output =
+                                template.render(&context.streaming(true)).await;
+                            // If this is a single-chunk template, we can might
+                            // be able to load directly from the source, since
+                            // we support file streams natively. In that case,
+                            // the stream will be thrown away
+                            let source = output.stream_source().cloned();
+                            let stream = output
+                                .try_into_stream()
+                                .map_err(|error| {
+                                    RequestBuildErrorKind::BodyFormFieldRender {
+                                        field: field.clone(),
+                                        error,
+                                    }
+                                })?
+                                .boxed();
+
+                            Ok::<_, RequestBuildErrorKind>((
+                                field.clone(),
+                                BodyStream { stream, source },
+                            ))
                         })
                     },
                 );
@@ -692,35 +737,80 @@ impl Authentication<String> {
 /// by which we'll add it to the request. This means it is **not** 1:1 with
 /// [RecipeBody]
 enum RenderedBody {
+    /// A body of plain ass bytes
     Raw(Bytes),
+    /// A body that should be streamed. If possible the value will be evaluated
+    /// lazily, meaning it won't be collected into bytes and instead will be
+    /// streamed over the network as the data becomes available. Only certain
+    /// data sources (such as files and commands) can be streamed.
+    Stream(BodyStream),
     /// JSON body
     Json(serde_json::Value),
     /// Field:value mapping. Value is `String` because only string data can be
     /// URL-encoded
     FormUrlencoded(Vec<(String, String)>),
-    /// Field:value mapping. Values can be arbitrary bytes
-    FormMultipart(Vec<(String, Vec<u8>)>),
+    /// Field:value mapping. Values can be arbitrary bytes or a binary stream
+    FormMultipart(Vec<(String, BodyStream)>),
 }
 
 impl RenderedBody {
-    fn apply(self, builder: RequestBuilder) -> RequestBuilder {
+    /// Add this body to the builder
+    async fn apply(
+        self,
+        builder: RequestBuilder,
+    ) -> Result<RequestBuilder, RequestBuildErrorKind> {
         // Set body. The variant tells us _how_ to set it
         match self {
-            RenderedBody::Raw(bytes) => builder.body(bytes),
-            RenderedBody::Json(json) => builder.json(&json),
-            RenderedBody::FormUrlencoded(fields) => builder.form(&fields),
+            RenderedBody::Raw(bytes) => Ok(builder.body(bytes)),
+            RenderedBody::Stream(stream) => {
+                let body = Body::wrap_stream(stream.stream);
+                Ok(builder.body(body))
+            }
+            RenderedBody::Json(json) => Ok(builder.json(&json)),
+            RenderedBody::FormUrlencoded(fields) => Ok(builder.form(&fields)),
             RenderedBody::FormMultipart(fields) => {
                 let mut form = Form::new();
-                for (field, value) in fields {
-                    let part = Part::bytes(value);
+
+                #[cfg(test)]
+                {
+                    // Hack alert!! Reqwest uses a random boundary between parts
+                    // in a multipart request. We share this with the tests via
+                    // TLS. See the TLS declaration for more info.
+                    tests::MULTIPART_BOUNDARY.set(form.boundary().to_owned());
+                }
+
+                for (field, stream) in fields {
+                    // Convert the stream to a form part
+                    let part = match stream.source {
+                        // Files can be handled natively by reqwest, which gets
+                        // bonus support for Content-Type and
+                        // Content-Disposition goodies
+                        Some(StreamSource::File { path }) => Part::file(path)
+                            .await
+                            .map_err(RequestBuildErrorKind::BodyFileStream)?,
+                        // Any other stream can be streamed directly as bytes
+                        _ => Part::stream(Body::wrap_stream(stream.stream)),
+                    };
                     form = form.part(field, part);
                 }
-                builder.multipart(form)
+
+                Ok(builder.multipart(form))
             }
         }
     }
 }
 
+/// A stream to be used in a request body
+struct BodyStream {
+    /// Stream of bytes
+    stream: BoxStream<'static, Result<Bytes, RenderError>>,
+    /// If the stream was generated by a template with a single dynamic
+    /// chunk (e.g. `{{ file('data.png') }}`), we can trace the source of
+    /// the stream. This allows certain applications to skip the stream and
+    /// handle the source natively.
+    source: Option<StreamSource>,
+}
+
 impl From<HttpMethod> for reqwest::Method {
     fn from(method: HttpMethod) -> Self {
         match method {
