@@ -3,7 +3,10 @@
 use crate::{
     collection::{HasId, ProfileId, RecipeId},
     database::CollectionDatabase,
-    http::{Exchange, HttpEngine, RequestSeed, TriggeredRequestError},
+    http::{
+        Exchange, HttpEngine, RequestSeed, StoredRequestError,
+        TriggeredRequestError,
+    },
     render::{HttpProvider, Prompt, Prompter, Select, TemplateContext},
 };
 use async_trait::async_trait;
@@ -25,7 +28,7 @@ pub fn invalid_utf8() -> Template {
 /// Create an HTTP engine for building/sending requests. We need to create a new
 /// engine for each test because each reqwest client is bound to a specific
 /// tokio runtime, and each test gets its own runtime.
-/// See https://github.com/LucasPickering/slumber/pull/524
+/// See <https://github.com/LucasPickering/slumber/pull/524>
 #[fixture]
 pub fn http_engine() -> HttpEngine {
     HttpEngine::new(&HttpEngineConfig {
@@ -61,9 +64,10 @@ impl HttpProvider for TestHttpProvider {
         &self,
         profile_id: Option<&ProfileId>,
         recipe_id: &RecipeId,
-    ) -> anyhow::Result<Option<Exchange>> {
+    ) -> Result<Option<Exchange>, StoredRequestError> {
         self.database
             .get_latest_request(profile_id.into(), recipe_id)
+            .map_err(StoredRequestError::new)
     }
 
     async fn send_request(
@@ -141,8 +145,10 @@ impl Prompter for TestSelectPrompter {
 
     fn select(&self, mut select: Select) {
         let index = self.index.fetch_add(1, Ordering::Relaxed);
-        if let Some(value) = self.responses.get(index) {
-            select.channel.respond(select.options.swap_remove(*value));
+        if let Some(value_index) = self.responses.get(index) {
+            select
+                .channel
+                .respond(select.options.swap_remove(*value_index).value);
         }
     }
 }
