@@ -11,9 +11,10 @@ use itertools::Itertools;
 use ratatui::{
     Frame,
     layout::{Alignment, Constraint, Layout},
-    text::{Line, Text},
+    text::{Line, Span},
 };
 use slumber_config::{Action, Config, InputBinding};
+use slumber_core::database::CollectionDatabase;
 use slumber_util::{doc_link, paths};
 
 const CRATE_VERSION: &str = env!("CARGO_PKG_VERSION");
@@ -24,7 +25,7 @@ pub struct HelpFooter;
 
 impl Generate for HelpFooter {
     type Output<'this>
-        = Text<'this>
+        = Span<'this>
     where
         Self: 'this;
 
@@ -44,7 +45,7 @@ impl Generate for HelpFooter {
             })
             .join(" / ");
 
-        Text::styled(text, tui_context.styles.text.highlight)
+        Span::styled(text, tui_context.styles.text.highlight)
     }
 }
 
@@ -103,12 +104,10 @@ impl Draw for HelpModal {
                 ("Log", paths::log_file().display().to_string().into()),
                 (
                     "Collection",
-                    ViewContext::with_database(|database| {
-                        database.collection_path()
-                    })
-                    .map(|path| path.display().to_string())
-                    .unwrap_or_default()
-                    .into(),
+                    ViewContext::with_database(CollectionDatabase::metadata)
+                        .map(|metadata| metadata.path.display().to_string())
+                        .unwrap_or_default()
+                        .into(),
                 ),
             ]
             .into_iter()
