@@ -1,14 +1,16 @@
 use crate::view::{
     Component,
-    common::{tabs::Tabs, template_preview::TemplatePreview},
+    common::tabs::Tabs,
     component::recipe_pane::{
         authentication::AuthenticationDisplay,
         body::RecipeBodyDisplay,
         persistence::RecipeOverrideKey,
         table::{RecipeFieldTable, RecipeFieldTableProps},
+        url::UrlDisplay,
     },
     draw::{Draw, DrawMetadata},
     event::{Child, EventHandler},
+    state::fixed_select::FixedSelectState,
     util::persistence::PersistedLazy,
 };
 use derive_more::Display;
@@ -18,6 +20,7 @@ use slumber_core::{
     collection::{Recipe, RecipeId},
     http::{BuildOptions, HttpMethod},
 };
+use std::iter;
 use strum::{EnumCount, EnumIter};
 
 /// Display a recipe. Not a recipe *node*, this is for genuine bonafide recipe.
@@ -26,8 +29,8 @@ use strum::{EnumCount, EnumIter};
 #[derive(Debug)]
 pub struct RecipeDisplay {
     tabs: Component<PersistedLazy<RecipeTabKey, Tabs<Tab>>>,
-    url: TemplatePreview,
     method: HttpMethod,
+    url: Component<UrlDisplay>,
     query: Component<RecipeFieldTable<QueryRowKey, QueryRowToggleKey>>,
     headers: Component<RecipeFieldTable<HeaderRowKey, HeaderRowToggleKey>>,
     body: Component<Option<RecipeBodyDisplay>>,
@@ -38,10 +41,26 @@ impl RecipeDisplay {
     /// Initialize new recipe state. Should be called whenever the recipe or
     /// profile changes
     pub fn new(recipe: &Recipe) -> Self {
+        // Disable tabs that have no content
+        let disabled_tabs = iter::empty()
+            .chain(recipe.body.is_none().then_some(Tab::Body))
+            .chain(
+                recipe
+                    .authentication
+                    .is_none()
+                    .then_some(Tab::Authentication),
+            );
+        let tabs = PersistedLazy::new(
+            RecipeTabKey,
+            Tabs::new(
+                FixedSelectState::builder().disabled(disabled_tabs).build(),
+            ),
+        );
+
         Self {
-            tabs: Default::default(),
+            tabs: tabs.into(),
             method: recipe.method,
-            url: TemplatePreview::new(recipe.url.clone(), None, false, false),
+            url: UrlDisplay::new(recipe.id.clone(), recipe.url.clone()).into(),
             query: RecipeFieldTable::new(
                 "Parameter",
                 QueryRowKey(recipe.id.clone()),
@@ -104,6 +123,7 @@ impl RecipeDisplay {
 
     /// Generate a [BuildOptions] instance based on current UI state
     pub fn build_options(&self) -> BuildOptions {
+        let url = self.url.data().override_value();
         let authentication = self.authentication.data().as_ref().and_then(
             super::authentication::AuthenticationDisplay::override_value,
         );
@@ -125,6 +145,7 @@ impl RecipeDisplay {
             .and_then(super::body::RecipeBodyDisplay::override_value);
 
         BuildOptions {
+            url,
             authentication,
             headers: self.headers.data().to_build_overrides(),
             query_parameters: self.query.data().to_build_overrides(),
@@ -138,6 +159,7 @@ impl EventHandler for RecipeDisplay {
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
         vec![
             self.tabs.to_child_mut(),
+            self.url.to_child_mut(),
             self.body.to_child_mut(),
             self.query.to_child_mut(),
             self.headers.to_child_mut(),
@@ -166,13 +188,14 @@ impl Draw for RecipeDisplay {
 
         // First line: Method + URL
         frame.render_widget(Paragraph::new(method), method_area);
-        frame.render_widget(&self.url, url_area);
+        frame.render_widget(self.url.data().preview(), url_area);
 
         // Navigation tabs
         self.tabs.draw(frame, (), tabs_area, true);
 
         // Recipe content
         match self.tabs.data().selected() {
+            Tab::Url => self.url.draw(frame, (), content_area, true),
             Tab::Body => self.body.draw_opt(frame, (), content_area, true),
             Tab::Query => self.query.draw(
                 frame,
@@ -218,6 +241,8 @@ struct RecipeTabKey;
 )]
 enum Tab {
     #[default]
+    #[display("URL")]
+    Url,
     Body,
     Query,
     Headers,
